/*--------------------------------------------------------------------------*/
// Program			: ibutton  
// Description 	: Read ID from ibutton Chip by 1-wire bus protocol
// Filename			: lab1001.c
// C complier		: RIDE 51 V6.1
/*--------------------------------------------------------------------------*/
#include <reg51.h>		// Header include register file
#include <onewire2051.h>		// Include Module function
#include <lcd2051.h>			// Module for Call function I/O serial port
#define delay(c) lcd_delay(c) 
/*****************************************************************************/
/*************** Function display character define n byte ********************/
/*****************************************************************************/
void lcd_nbyte(unsigned char line,char *p,unsigned char count)
{
	unsigned char m;
	lcd_origin();         		// Set origin address of LCD
	lcd_command(line);			// Set address 00H of LCD
	for(m=0;m<count;m++)
	{
	 		lcd_text(*p++);      // Send data to LCD
	}
}
/*****************************************************************************/
/************************ Function swap data *********************************/
/*****************************************************************************/
void swap_byte(unsigned char *byte1,unsigned char *byte2)
{
	unsigned char x;		// For buffer
	x = *byte2;          // Keep data to buffer
	*byte2 = *byte1;     // Swap byte
	*byte1 = x;          // Load data from buffer
}
/*****************************************************************************/
/*************** Function Display CODE for DS1990A ***************************/
/*****************************************************************************/
void ID_display(char *dis)
{
	lcd_puts(line1,"F-Code: "); 	// Screen for Family code
	lcd_nbyte(0x87,dis,2);			// Display Family code
	lcd_puts(0x8A,"CRC: ");       // Screen for CRC code
	lcd_nbyte(0x8E,dis+14,2);		// Display CRC code
	lcd_puts(line2,"ID: ");       // Screen for ID code
	lcd_nbyte(0xC4,dis+2,12);		// Display ID code
}
/*****************************************************************************/
/*************** Function Display message for Ready ***************************/
/*****************************************************************************/
void Ready_display(void)
{
	lcd_clear();               // Clear display 
	lcd_puts(0x84,"Ready!");   // Display message
	lcd_puts(line2,"Insert Your Key!");	// Display message
}
/*****************************************************************************/
/*************** Function Display message for initial ************************/
/*****************************************************************************/
void Demo_display(void)
{
	lcd_clear();        					// Clear display 
	lcd_puts(0x84,"PROGRAM");			// Display message
	lcd_puts(0xC2,"ibutton Demo");	// Display message
	delay(5000);	// Delay 5 sec
}
/*****************************************************************************/
/*************** Function Write data to 1-wire bus ***************************/
/*****************************************************************************/
void inttoascii(char *con, char *dat)
{
		char i,buf;     // For buffer
		
		for(i=0;i<8;i++)     // Loop for keep data to array 8 byte
		{
	 	 	buf = ((*(dat+i)) & 0xF0)>>4;		// Filter for bit7-bit4 
	 	 	if(buf>=0x0A)
	 	 	{*(con+(i*2)) = (buf-9) | 0x40;}	// Convert value 0x0A-0x0F to ascii 
	 	 	else
	 	 	{*(con+(i*2)) = buf | 0x30;}		// Convert value 0x00-0x09 to ascii
	 		
	 		buf = ((*(dat+i)) & 0x0F);      	// Filter for bit3-bit0
	 		if(buf>=0x0A)
	 	 	{*(con+(i*2)+1) = (buf-9) | 0x40;} 	// Convert value 0x0A-0x0F to ascii
	 	 	else
	 	 	{*(con+(i*2)+1) = buf | 0x30;}	// Convert value 0x00-0x09 to ascii			 
		}

} 
/*****************************************************************************/
/************************** Main Program *************************************/
/*****************************************************************************/
void main()
{        
	char id_read[8],id_con[16],n = 0;   	// For buffer
	
	lcd_init();   		// initial LCD
	Demo_display();   // Display Title 
	Ready_display();	// Display message for ready
	while(1)          // Infinite loop
	{
	//--------- Connect and Read ID ibutton by 1-wire protocal --------------//
		onewire_reset();		// DS1990A Reset 
		onewire_ans();       // DS1990A answer  
		onewire_write(0x33);	// Write Read Rom DS1990A
		for(n=0;n<8;n++)     // Loop for keep data to array 8 byte
		{
	 		id_read[n] = onewire_read();  // Keep data to id_read array
		}
		//---------------------------Check Family Code------------------------//	
		if(id_read[0]==0x01)
		{
			swap_byte(&id_read[1],&id_read[6]);	// Swap byte between id_read[1] and id_read[6]
			swap_byte(&id_read[2],&id_read[5]);	// Swap byte between id_read[2] and id_read[5]
			swap_byte(&id_read[3],&id_read[4]);	// Swap byte between id_read[3] and id_read[4]
			inttoascii(&id_con,&id_read);		// Convert to ascii
			lcd_clear();   		// Clear LCD
			ID_display(&id_con); // Display ID DS1990A
			delay(5000); 			// Dealy 5 sec
	   }
	   Ready_display();			// Display message for ready   
	}
		
}
/**********************************************************************************/






















