/*--------------------------------------------------------------------------*/
// Program			: Read Temp.  
// Description 	: Read Temp. and show result on LCD 
// Filename			: lab1201.c
// C compiler		: RIDE 51 V6.1
/*--------------------------------------------------------------------------*/
#include<reg51.h>				// Header include register of P89C51RD2 
#include<i2c2051.h>					// Inclued for Call function i2c bus communication
#include<lcd2051.h>					// Module function LCD display
#define DS1621_ID 0x90       	// Difine constant for AT24C16 ID
data unsigned char flag=0,ser_command=0,ser_temp=0;   	// for check .5 or .0 
//code unsigned char *scr1 = "DS1621 Read...";
//code unsigned char *scr2 = "Temp:    .  C";
/***********************************************************************************/
/******************************** Initial Start convert ****************************/
/***********************************************************************************/
void ds1621_init() 
{
   i2c_start();					// i2c start condition
   i2c_wrdata(DS1621_ID);     // Write DS1307 ID for connect 
   i2c_wrdata(0xee);          // Write command convert Temp.
   i2c_stop();						// i2c stop condition
}
/***********************************************************************************/
/******************************** Convert to LCD display ***************************/
/***********************************************************************************/
void temptolcd(unsigned char tempdat)
{
	unsigned	char tempstr[7],i=4;
	bit sign_neg = 0;	// Define bit for Check negative temp and initial zero value   
	//---------------------------Check negative value-----------------------------//
	if(tempdat>=128)	// If temp negative value  
	{
		sign_neg = 1; 	//	Set bit sign_neg for temp negative value 
		tempdat = (tempdat^0xFF)+1;	// Convert temp by 2'complement
	}
	//---------------------------Convert tempdata to ASCII------------------------//
	do 	// do-while loop for convert temp keep to array tempstr
	{		
		tempstr[--i] = (tempdat%10) + '0';	// Convert to ASCII
		tempdat /= 10; 		// Div for Next digit
	}while(tempdat != 0);	// Check Convert success
	//---------------------------Past sign and space------------------------------//
	if(sign_neg)		
	{
		tempstr[--i] = '-';	// Keep negative sign for temp is negative value
	}	
	while(i>0)           // While loop for keep space in address
	{
		tempstr[--i] = 0x20;		// Keep space to array
	}
	//---------------------------Section display .0 and .5------------------------//
	tempstr[4] = '.'; 	// Past "." for dot display on lcd
	if(flag)             // Check decimal point
	{
		tempstr[5] = '5';		// For .5 display 
	}			
	else
	{
		tempstr[5] = '0'; 	// For .5 display
	}
	//---------------------------Stop of string----------------------------------//
	tempstr[6]= 0;					// Push NULL
	//---------------------------Send temp data to LCD---------------------------//
	lcd_puts(0xC5,&tempstr);	// Send temp convert 
}
/***********************************************************************************/
/******************************** Read Temp. from ds1621 ***************************/
/***********************************************************************************/
char read_temp() 
{        
   data unsigned char datah,datal;	// For keep Temp.
   i2c_start();					// i2c start condition
   i2c_wrdata(DS1621_ID);     // Write DS1307 ID for connect  
   i2c_wrdata(0xaa);				// Write command read Temp.
   i2c_start();					// i2c start condition
   i2c_wrdata(DS1621_ID+1);	// Write DS1307 ID for read data condition
   datah=i2c_rddata();        // Read data to keep datah
   i2c_ACK();						// i2c acknowlage	
   datal=i2c_rddata();			// Read data to keep datal
   i2c_NACK();						// i2c acknowlage
   i2c_stop();               	// i2c stop condition
   if((datal & 0x80)==0x80)   // Check Temp. .5 or .0 keep result by flag variable
   {flag = 1;}
   else{flag = 0;}   
  	return(datah);             // Return Temp.
}
/***********************************************************************************/
/******************************** Show screen display ******************************/
/***********************************************************************************/
void display_screen()
{
	lcd_puts(line1,"DS1621 Read...");	//	Show message
	lcd_puts(line2,"Temp:    .  C");		//	Show message
}
/***********************************************************************************/
/************************* Service interrupt serial port ***************************/
/***********************************************************************************/
void service_serial(void) interrupt 4
{
	if(RI)
	{
		ser_command = SBUF;
		RI = 0;
	}
	if(TI)
	{
		TI = 0;
	}
}
/***********************************************************************************/
/******************************** Main Promgram ************************************/
/***********************************************************************************/
void main()
{
  	unsigned temp=0;     // Keep temp
  	ds1621_init();   		// Initial for send command DS1620
	lcd_init();       	// Initial for lcd module
	display_screen();  	// For display screen at LCD display 
	//-------------- Enable interrupt serial port configuration 9600 bps ------------//
	SCON = 0x50;         // Configuration for serial port Rx/Tx data
	TMOD = 0x21;			// Timer 1 mode 2(8 bit auto reload for serial baudrate) 
	TH1 = 0xFD;				// Reload value for serial baudrate(9600 bps)
	TL1 = 0xFD;
	EA = 1;					// Enable global interrupt
	ES = 1;					// Enable serial interrupt
	TR1 = 1;             // Start timer1 for serial port communication
	//-------------------------------------------------------------------------------//
	while(1)							// Infinite loop
	{
		temp = read_temp();	// Read temp	
		ser_temp = temp;     // Keep temp to ser_temp
		temptolcd(temp);  	// Read Temp. and convert to LCD display
		//-------------------------- Send positive temp ------------------------------//
		if(ser_command=='i')	// Check command "i" ? 
		{
			SBUF = ser_temp;   // Send temp data integer
			ser_command = 0;   // Clear command
		}
		else if(ser_command=='f')	// Check command "f" ?
		{
			SBUF = flag;        // Send data for .5 or .0 to serial
			ser_command = 0;	// Clear command
		}
		//----------------------------------------------------------------------------//
	}
} 
/***********************************************************************************/






