/*--------------------------------------------------------------------------*/
// Program			: Module function i2c bus communication 
// Description 	: Call function for i2c bus communication  
// Filename			: i2c2051.h
// C compiler		: RIDE 51 V6.1
/*--------------------------------------------------------------------------*/
#include<intrins.h>		// For call nop function
sbit SDA = P1^1;        // Bit data for i2c bus
sbit SCL = P1^0;			// Bit clock for i2c bus
/**********************************************************************/
/****************** Function for delay time ***************************/
/**********************************************************************/
void i2c_delay(void)
{
	unsigned char i;  	// For keep counter
	for(i=0;i<10;i++)    // For loop delay of i2c bus
	_nop_();	            // Call function  from "intrins.h"
}
/**********************************************************************/
/****************** Function for generate clock signal ****************/
/**********************************************************************/
void i2c_clk(void)
{
	i2c_delay();       	// Delay
	SCL = 1;             // For start clock
	i2c_delay();         // Delay
	SCL = 0;             // End of clock
}
/**********************************************************************/
/****************** Function for start condition **********************/
/**********************************************************************/
void i2c_start(void)
{
	if(SCL)          		// If SCL=1 clear SCL
	SCL = 0;             // Clear SCL
	SDA = 1;             // Set SDA
	SCL = 1;             // Set SCL
	i2c_delay();         // Delay
	SDA = 0;            	// Clear SDA
	i2c_delay();			// Delay
	SCL = 0;					// Clear SCL
}
/**********************************************************************/
/****************** Function for stop condition ***********************/
/**********************************************************************/
void i2c_stop(void)
{
	if(SCL)           	// If SCL=1 clear SCL 
	SCL = 0;         		// Clear SCL
	SDA = 0;            	// Clear SDA
	i2c_delay();   		// Delay
	SCL = 1;           	// Set SCL
	i2c_delay();      	// Delay
	SDA = 1;             // Set SDA
}
/**********************************************************************/
/****************** Function for write data ***************************/
/**********************************************************************/
bit i2c_wrdata(unsigned char dat)
{
	bit data_bit;			// Bit keep data send to i2c bus
	unsigned char i;     // For keep counter
	for(i=0;i<8;i++)  	// For loop send data 1 byte(8 time)
	{
		data_bit = dat & 0x80;		// Filter MSB bit keep to data_bit
		SDA = data_bit;         	// Send data_bit to SDA
		i2c_clk();                 // Call for send data to i2c bus
		dat = dat<<1;              // Shift left 1 time
	}
	SDA = 1;      				// Set SDA      			
	i2c_delay();         	// Delay
	SCL = 1;             	// Set SCL
	i2c_delay();           	// Delay
	data_bit = SDA;      	// For Check acknowledge
	SCL = 0;                // Clear SCL
	i2c_delay();				// Delay
	return(data_bit);			// if send_bit = 0 i2c OK! 	
}
/**********************************************************************/
/****************** Function for read data ****************************/
/**********************************************************************/
unsigned char i2c_rddata(void)
{
	bit rd_bit;					// Bit read data for i2c bus
	unsigned char i,dat;   	// For keep counter and data receive
	dat = 0x00;            	// Clear data
	for(i=0;i<8;i++)        // For loop read data 1 byte(8 time) 
	{
		i2c_delay();      	// Delay
		SCL = 1;           	// Set SCL
		i2c_delay();        	// Delay
		rd_bit = SDA;       	// Keep for check acknowledge
		dat = dat<<1;       	// Shift  left 1 time
		dat = dat | rd_bit;	// Keep bit data in dat
		SCL = 0;           	// Clear SCL
	}
	return(dat);         	// Return data read form i2c bus
}
/**********************************************************************/
/****************** Function for acknowledge ***************************/
/**********************************************************************/
void i2c_ACK()
{
	SDA = 0;        			// Clear SDA
	i2c_delay();           	// Delay
	i2c_clk();           	// Call for send data to i2c bus 	
	SDA = 1;                // Set SDA
}
/**********************************************************************/
/****************** Function for no acknowledge ***********************/
/**********************************************************************/
void i2c_NACK()
{
	SDA = 1;						// Set SDA
	i2c_delay();				// Delay
	i2c_clk();           	// Call for send data to i2c bus
	SCL = 1;						// Set SCL
}






