/*--------------------------------------------------------------------------*/
// Program			: Module function 1-wire bus communication 
// Description 	: Call function for 1-wire bus communication  
// Filename			: onewire2051.h
// C compiler		: RIDE 51 V6.1
/*--------------------------------------------------------------------------*/
#include <intrins.h>       // Module function for call nop function
sbit onewire = P3^7;			// Bit data 1-wire bus
unsigned int i;				// Counter for delay time (for 1-wire bus)
/*****************************************************************************/
/*************** Function 1-wire Start condition *****************************/
/*****************************************************************************/
void onewire_reset()
{	
  	onewire = 0;						// Clear bit 1-wire
	for(i=0;i<100;i++) _nop_(); 	//Delay 800 microsec  {while(i>0) i--; ==> 8 machine cycle}
	onewire = 1;						// Set bit 1-wire
	for(i=0;i<2;i++) _nop_();		//Delay 16 microsec
}
/*****************************************************************************/
/*************** Function Check 1-wire acknowledge ***************************/
/*****************************************************************************/
void onewire_ans(void)
{  
   while(onewire);					// Wait 1-wire = 1
	while(~onewire);					// Wait 1-wire = 0
	for(i=0;i<2;i++) _nop_();		// Delay 16 microsec
}
/*****************************************************************************/
/*************** Function Read data 1 time bit *******************************/
/*****************************************************************************/
bit readbit (void)
{	
 	bit dat;					// Bit for keep data
	onewire = 0;        	// Clear bit 1-wire
	_nop_();             // Delay
	_nop_();					// Delay
	_nop_();             // Delay
	_nop_();					// Delay
	onewire = 1;			// Set bit 1-wire
	_nop_();          	// Delay
	_nop_();					// Delay
	_nop_();					// Delay
	_nop_(); 				// Delay
	_nop_();					// Delay
	_nop_(); 				// Delay

	dat = onewire;       // Read data bit keep in dat
   for(i=0;i<8;i++) _nop_();		// Delay 64 microsec
	return(dat);        	// Return dat 
}
/*****************************************************************************/
/*************** Function Read data 1 byte(Temperature) **********************/
/*****************************************************************************/
unsigned char onewire_read()
{  
	unsigned char i,j,dat;		// For counter and keep data
	dat = 0;                   // Clear data
	for(i=0;i<8;i++)           // For loop Read data 1 byte(8 time) 
	{
     	j = readbit();          // Keep data bit to j
		dat = (j<<7)|(dat>>1);  // Shift left j 7 time OR   
	}
	return(dat);					// Return dat
}
/*****************************************************************************/
/*************** Function Write data to 1-wire bus ***************************/
/*****************************************************************************/
void onewire_write(unsigned char com)
{
	unsigned char j;			// For couter  
	bit send;               // Bit send data 1-wire
  	for(j=0;j<8;j++)        // For loop Write data 1 byte(8 time) 
	{
     	send = com & 0x01;   // LSB bit keep to send
		com = com>>1;        // Shift Rigth 1 time
		if(send)             // Check send = 1?
		{
        	onewire = 0;	// Clear bit 1-wire
        	_nop_(); 		// Delay  delay 4 microsec
        	_nop_();			// Delay
        	_nop_();    	// Delay
        	_nop_();			// Delay
			onewire = 1;	// Set bit 1-wire
			for(i=0;i<8;i++) _nop_();	//delay 64 microsec
      }
      else
		{
        	onewire = 0;	// Clear bit 1-wire
			for(i=0;i<8;i++) _nop_(); // delay 64 microsec
			onewire = 1;	// Set bit 1-wire
			_nop_();			// Delay  delay 4 microsec
        	_nop_();			// Delay
        	_nop_();			// Delay
        	_nop_();			// Delay
      }
  	}
}
 
